import asyncio
from typing import List, cast

from cookit import flatten
from cookit.loguru import logged_suppress
from httpx import AsyncClient, HTTPStatusError
from nonebot_plugin_alconna.uniseg import UniMessage
from PicImageSearch import Ascii2D
from PicImageSearch.model import Ascii2DResponse

from ..config import config
from ..registry import SearchFunctionReturnType, search_function
from ..utils import async_lock, combine_message, get_image_bytes_by_url, shorten_url


@search_function("a2d")
@async_lock()
async def ascii2d_search(
    file: bytes,
    client: AsyncClient,
    _: str,
) -> SearchFunctionReturnType:
    ascii2d_color = Ascii2D(client=client)
    color_res = await ascii2d_color.search(file=file)
    if not color_res.raw:
        return [UniMessage.text("Ascii2D 暂时无法使用")]

    resp_text, resp_url, _ = await ascii2d_color.get(
        color_res.url.replace("/color/", "/bovw/"),
    )
    bovw_res = Ascii2DResponse(resp_text, resp_url)

    res = await asyncio.gather(
        get_final_res(color_res),
        get_final_res(bovw_res, bovw=True),
    )
    return flatten(res)


async def get_final_res(res: Ascii2DResponse, bovw: bool = False) -> List[UniMessage]:
    final_res_list: List[UniMessage] = []
    for r in res.raw:
        if not (r.title or r.url_list):
            continue

        msg = UniMessage()
        if config.hide_img:
            msg += f"预览图链接：{r.thumbnail}\n"
        else:
            thumbnail = None
            with logged_suppress("Failed to get image", HTTPStatusError):
                thumbnail = await get_image_bytes_by_url(r.thumbnail)
            if not thumbnail:
                continue
            msg += UniMessage.image(raw=thumbnail)
            msg += "\n"

        title = r.title
        if r.url_list and title == r.url_list[0].text:
            title = ""

        source = r.url or (cast(str, r.url_list[0].href) if r.url_list else "")
        source = await shorten_url(source) if source else ""

        author = r.author
        if author and r.author_url:
            author_url = await shorten_url(r.author_url)
            author = f"[{author}]({author_url})"

        res_list = [
            r.detail,
            title,
            f"作者：{author}" if author else "",
            f"来源：{source}" if source else "",
        ]
        msg += combine_message(res_list)

        final_res_list.append(msg)
        if len(final_res_list) == 3:
            break

    return [
        UniMessage.text(
            f"Ascii2D {'特徴' if bovw else '色合'}検索結果\n"
            f"搜索页面：{await shorten_url(res.url)}",
        ),
        *final_res_list,
    ]



from typing import List, Optional, Set

from cookit.pyd import field_validator
from nonebot import get_plugin_config
from pydantic import BaseModel, Field


class ConfigModel(BaseModel):
    nickname: Set[str]

    proxy: Optional[str] = None

    saucenao_api_key: str
    exhentai_cookies: Optional[str] = None
    nhentai_useragent: Optional[str] = None
    nhentai_cookies: Optional[str] = None

    saucenao_low_acc: int = 60
    auto_use_ascii2d: bool = True

    search_keyword: str = "搜图"
    search_keyword_only: bool = False
    search_in_group_only_keyword: bool = True
    search_immediately: bool = True
    wait_for_image_timeout: int = 180

    hide_img: bool = False
    hide_img_when_low_acc: bool = True
    hide_img_when_whatanime_r18: bool = True
    saucenao_nsfw_hide_level: int = Field(2, ge=0, le=3)
    forward_search_result: bool = True
    to_confuse_urls: List[str] = [
        "ascii2d.net",
        "danbooru.donmai.us",
        "konachan.com",
        "pixiv.net",
        "saucenao.com",
        "yandex.com",
    ]

    cache_expire: int = 3

    @field_validator("saucenao_api_key", mode="before")
    def saucenao_api_key_validator(cls, v: str) -> str:  # noqa: N805
        if not v:
            raise ValueError("请配置 SAUCENAO_API_KEY 否则无法正常使用搜图功能！")
        return v

    @field_validator("proxy", mode="before")
    def proxy_validator(cls, v: Optional[str]) -> Optional[str]:  # noqa: N805
        if isinstance(v, str) and v.startswith("socks://"):
            raise ValueError(
                '请修改代理地址为 "socks5://" 或 "socks4://" 的格式，具体取决于你的代理协议！',
            )
        return v


config = get_plugin_config(ConfigModel)



import re
from typing import List

from httpx import AsyncClient
from nonebot_plugin_alconna.uniseg import UniMessage
from PicImageSearch import SauceNAO
from PicImageSearch.model import SauceNAOItem, SauceNAOResponse

from ..config import config
from ..registry import (
    SearchFunctionReturnTuple,
    SearchFunctionReturnType,
    search_function,
)
from ..utils import (
    async_lock,
    combine_message,
    get_source,
    get_valid_url,
    handle_img,
    shorten_url,
)
from .ascii2d import ascii2d_search
from .ehentai import ehentai_title_search
from .nhentai import nhentai_title_search
from .whatanime import whatanime_search

SAUCENAO_DB = {
    "all": 999,
    "pixiv": 5,
    "danbooru": 9,
    "anime": [21, 22],
    "doujin": [18, 38],
    "fakku": 16,
}


@search_function(*SAUCENAO_DB.keys())
@async_lock(freq=8)
async def saucenao_search(
    file: bytes,
    client: AsyncClient,
    mode: str,
) -> SearchFunctionReturnType:
    db = SAUCENAO_DB[mode]
    if isinstance(db, list):
        saucenao = SauceNAO(
            client=client,
            api_key=config.saucenao_api_key,
            hide=config.saucenao_nsfw_hide_level,
            dbs=db,
        )
    else:
        saucenao = SauceNAO(
            client=client,
            api_key=config.saucenao_api_key,
            hide=config.saucenao_nsfw_hide_level,
            db=db,
        )
    res = await saucenao.search(file=file)

    if (
        res
        and res.status == 429
        and "4 searches every 30 seconds" in res.origin["header"]["message"]
    ):
        return await saucenao_search(file, client, mode)

    if not res or not res.raw:
        final_res = [
            UniMessage.text("SauceNAO 暂时无法使用，自动使用 Ascii2D 进行搜索"),
        ]
        return final_res, ascii2d_search

    selected_res = get_best_result(res, res.raw[0])
    return await get_final_res(mode, res, selected_res)


def get_best_pixiv_result(
    res: SauceNAOResponse,
    selected_res: SauceNAOItem,
) -> SauceNAOItem:
    pixiv_res_list = list(
        filter(
            lambda x: x.index_id == SAUCENAO_DB["pixiv"]
            and x.url
            and abs(x.similarity - selected_res.similarity) < 5,
            res.raw,
        ),
    )
    if len(pixiv_res_list) > 1:
        selected_res = min(
            pixiv_res_list,
            key=lambda x: int(re.search(r"\d+", x.url).group()),  # type: ignore
        )
    return selected_res


def get_best_result(res: SauceNAOResponse, selected_res: SauceNAOItem) -> SauceNAOItem:
    # 如果结果为 pixiv ，尝试找到原始投稿，避免返回盗图者的投稿
    if selected_res.index_id == SAUCENAO_DB["pixiv"]:
        selected_res = get_best_pixiv_result(res, selected_res)
    # 如果地址有多个，优先取 danbooru
    elif len(selected_res.ext_urls) > 1:
        for i in selected_res.ext_urls:
            if "danbooru" in i:
                selected_res.url = i
    return selected_res


async def get_final_res(
    mode: str,
    res: SauceNAOResponse,
    selected_res: SauceNAOItem,
) -> SearchFunctionReturnType:
    low_acc = selected_res.similarity < config.saucenao_low_acc
    hide_img = bool(
        config.hide_img
        or selected_res.hidden
        or (low_acc and config.hide_img_when_low_acc),
    )

    thumbnail = await handle_img(selected_res.thumbnail, hide_img)

    url = await shorten_url(selected_res.url)
    source = selected_res.source if selected_res.source != selected_res.title else ""
    if not source and selected_res.url:
        source = await get_source(selected_res.url)
    if source and get_valid_url(source):
        source = await shorten_url(source)

    author_link = (
        f"[{selected_res.author}]({await shorten_url(selected_res.author_url)})"
        if selected_res.author and selected_res.author_url
        else ""
    )

    res_list = [
        f"SauceNAO ({selected_res.similarity}%)",
        thumbnail,
        selected_res.title,
        f"作者：{author_link}" if author_link else "",
        url if url != source else "",
        f"来源：{source}" if source else "",
        f"搜索页面：{res.url}",
    ]

    final_res: List[UniMessage] = []

    if res.long_remaining and res.long_remaining < 10:
        final_res.append(
            UniMessage.text(f"SauceNAO 24h 内仅剩 {res.long_remaining} 次使用次数"),
        )

    final_res.append(combine_message(res_list))

    if low_acc:
        extra_res, extra_handle = await handle_saucenao_low_acc(mode, selected_res)
        final_res.extend(extra_res)
        return final_res, extra_handle
    if selected_res.index_id in SAUCENAO_DB["doujin"]:  # type: ignore
        title = selected_res.title.replace("-", "")
        final_res.extend(await search_on_ehentai_and_nhentai(title))
    # 如果搜索结果为 fakku ，额外返回 ehentai 的搜索结果
    elif selected_res.index_id == SAUCENAO_DB["fakku"]:
        title = f"{selected_res.author} {selected_res.title}"
        final_res.extend(await search_on_ehentai_and_nhentai(title))
    elif selected_res.index_id in SAUCENAO_DB["anime"]:  # type: ignore
        return final_res, whatanime_search

    return final_res, None


async def search_on_ehentai_and_nhentai(title: str) -> List[UniMessage]:
    title_search_result = await ehentai_title_search(title)

    if (
        title_search_result[0].startswith("EHentai 搜索结果为空")
        and config.nhentai_useragent
        and config.nhentai_cookies
    ):
        nhentai_title_search_result = await nhentai_title_search(title)
        if not nhentai_title_search_result[0].startswith("NHentai 搜索结果为空"):
            title_search_result = nhentai_title_search_result

    return title_search_result


async def handle_saucenao_low_acc(
    mode: str,
    selected_res: SauceNAOItem,
) -> SearchFunctionReturnTuple:
    final_res: List[UniMessage] = []
    # 因为 saucenao 的动画搜索数据库更新不够快，所以当搜索模式为动画时额外增加 whatanime 的搜索结果
    if mode == "anime":
        return final_res, whatanime_search
    if config.auto_use_ascii2d:
        final_res.append(
            UniMessage.text(
                f"相似度 {selected_res.similarity}% 过低，自动使用 Ascii2D 进行搜索",
            ),
        )
        return final_res, ascii2d_search

    return final_res, None



from dataclasses import dataclass
from typing import Awaitable, Callable, Dict, List, Optional, Tuple, TypeVar, Union
from typing_extensions import TypeAlias

from httpx import AsyncClient
from nonebot_plugin_alconna.uniseg import UniMessage

SearchFunctionReturnTuple: TypeAlias = Tuple[
    List[UniMessage],
    Optional["SearchFunctionType"],
]
SearchFunctionReturnType: TypeAlias = Union[List[UniMessage], SearchFunctionReturnTuple]
SearchFunctionType: TypeAlias = Callable[
    [bytes, AsyncClient, str],
    Awaitable[SearchFunctionReturnType],
]

TSF = TypeVar("TSF", bound=SearchFunctionType)


@dataclass
class SearchFunctionInfo:
    func: SearchFunctionType


registered_search_func: Dict[str, SearchFunctionInfo] = {}


def search_function(*modes: str):
    def deco(func: TSF) -> TSF:
        info = SearchFunctionInfo(func)
        for mode in modes:
            registered_search_func[mode] = info
        return func

    return deco



import itertools
import re
from collections import defaultdict
from typing import Any, Dict, List, cast

import arrow
from httpx import AsyncClient
from nonebot_plugin_alconna.uniseg import UniMessage
from PicImageSearch import EHentai
from PicImageSearch.model import EHentaiResponse
from pyquery import PyQuery

from ..config import config
from ..registry import SearchFunctionReturnType, search_function
from ..utils import (
    DEFAULT_HEADERS,
    async_lock,
    combine_message,
    filter_results_with_ratio,
    handle_img,
    parse_cookies,
    preprocess_search_query,
    shorten_url,
)
from .ascii2d import ascii2d_search


@search_function("ex")
@async_lock(freq=8)
async def ehentai_search(
    file: bytes,
    client: AsyncClient,
    mode: str,
) -> SearchFunctionReturnType:
    ex = bool(config.exhentai_cookies)
    ehentai = EHentai(client=client)

    if not (res := await ehentai.search(file=file, ex=ex)):
        if "Please wait a bit longer between each file search" in res.origin:
            return await ehentai_search(file, client, mode)

        final_res = await search_result_filter(res)
        if not res.raw and config.auto_use_ascii2d:
            final_res.append(UniMessage.text("自动使用 Ascii2D 进行搜索"))
            return final_res, ascii2d_search

        return final_res

    return [UniMessage.text("EHentai 暂时无法使用")]


async def ehentai_title_search(title: str) -> List[UniMessage]:
    query = preprocess_search_query(title)
    url = "https://exhentai.org" if config.exhentai_cookies else "https://e-hentai.org"
    params: Dict[str, Any] = {"f_search": query}

    async with AsyncClient(
        headers=DEFAULT_HEADERS,
        cookies=parse_cookies(config.exhentai_cookies),
        proxies=config.proxy,
    ) as session:
        resp = await session.get(url, params=params)
        if res := EHentaiResponse(resp.text, str(resp.url)):
            if not res.raw:
                # 如果第一次没找到，使搜索结果包含被删除的部分，并重新搜索
                params["advsearch"] = 1
                params["f_sname"] = "on"
                params["f_sh"] = "on"
                resp = await session.get(url, params=params)
                res = EHentaiResponse(resp.text, str(resp.url))

            # 只保留标题和搜索关键词相关度较高的结果，并排序，以此来提高准确度
            if res.raw:
                res.raw = filter_results_with_ratio(res, title)
            return await search_result_filter(res)

        return [UniMessage.text("EHentai 暂时无法使用")]


async def search_result_filter(
    res: EHentaiResponse,
) -> List[UniMessage]:
    url = await shorten_url(res.url)
    if not res.raw:
        return [UniMessage.text(f"EHentai 搜索结果为空\n搜索页面：{url}")]

    # 尝试过滤已删除的
    if not_expunged_res := [
        i for i in res.raw if not PyQuery(i.origin)("[id^='posted'] s")
    ]:
        res.raw = not_expunged_res

    # 尝试过滤无主题的杂图图集
    if not_themeless_res := [i for i in res.raw if "themeless" not in " ".join(i.tags)]:
        res.raw = not_themeless_res

    # 尝试过滤评分低于 3 星的
    if above_3_star_res := [
        i
        for i in res.raw
        if get_star_rating(cast(str, PyQuery(i.origin)("div.ir").attr("style"))) >= 3
    ]:
        res.raw = above_3_star_res

    # 尽可能过滤掉非预期结果(大概
    priority = defaultdict(lambda: 0)
    priority["Image Set"] = 1
    priority["Non-H"] = 2
    priority["Western"] = 3
    priority["Misc"] = 4
    priority["Cosplay"] = 5
    priority["Asian Porn"] = 6
    res.raw.sort(key=lambda x: priority[x.type], reverse=True)
    for key, group in itertools.groupby(res.raw, key=lambda x: x.type):  # type: ignore
        if priority[key] > 0:
            group_list = list(group)
            if len(res.raw) != len(group_list):
                res.raw = [i for i in res.raw if i not in group_list]

    # 优先找汉化版；没找到就优先找原版
    if chinese_res := [
        i
        for i in res.raw
        if "translated" in " ".join(i.tags) and "chinese" in " ".join(i.tags)
    ]:
        selected_res = chinese_res[0]
    elif not_translated_res := [
        i for i in res.raw if "translated" not in " ".join(i.tags)
    ]:
        selected_res = not_translated_res[0]
    else:
        selected_res = res.raw[0]

    thumbnail = await handle_img(
        selected_res.thumbnail,
        cookies=config.exhentai_cookies,
    )
    date = arrow.get(selected_res.date).to("Asia/Shanghai").format("YYYY-MM-DD HH:mm")
    favorited = bool(selected_res.origin.find("[id^='posted']").eq(0).attr("style"))
    res_list = [
        "EHentai 搜索结果",
        thumbnail,
        selected_res.title,
        ("❤️ 已收藏" if favorited else ""),
        f"类型：{selected_res.type}",
        f"日期：{date}",
        f"来源：{selected_res.url}",
        f"搜索页面：{url}",
    ]
    return [combine_message(res_list)]


def get_star_rating(css_style: str) -> float:
    x, y = re.search(r"(-?\d+)px (-\d+)px", css_style).groups()  # type: ignore
    star_rating = 5 - int(x.rstrip("px")) / -16
    if y == "-21px":
        star_rating -= 0.5
    return star_rating



# ruff: noqa: N999

import asyncio
from contextlib import asynccontextmanager, suppress
from dataclasses import dataclass
from pathlib import Path
from typing import List, Literal, NoReturn, Optional, Union, overload

from cookit.loguru import logged_suppress
from cookit.nonebot.alconna import RecallContext
from httpx import AsyncClient
from nonebot import logger, on_command, on_message
from nonebot.adapters import Bot as BaseBot, Event as BaseEvent, Message as BaseMessage
from nonebot.exception import ActionFailed, FinishedException
from nonebot.matcher import current_bot, current_event, current_matcher
from nonebot.params import _command, _command_arg
from nonebot.permission import SUPERUSER
from nonebot.typing import T_State
from nonebot_plugin_alconna.uniseg import (
    At,
    CustomNode,
    FallbackStrategy,
    Image,
    MsgTarget,
    Reference,
    Reply,
    SerializeFailed,
    Target,
    Text,
    UniMessage,
    UniMsg,
)
from nonebot_plugin_waiter import waiter
from PicImageSearch import Network

from .cache import msg_cache
from .config import config
from .registry import registered_search_func
from .utils import get_image_from_seg, post_image_process

KEY_IMAGES = "images"


@dataclass
class SearchArgs:
    mode: str = "all"
    purge: bool = False


async def extract_images(msg: UniMsg) -> List[Image]:
    if Reply in msg and isinstance((raw_reply := msg[Reply, 0].msg), BaseMessage):
        msg = await UniMessage.generate(message=raw_reply)
    return msg[Image]


async def rule_func_search_msg(
    bot: BaseBot,
    ev: BaseEvent,
    state: T_State,
    msg: UniMsg,
    target: MsgTarget,
) -> bool:
    if target.private:
        images = await extract_images(msg)
        state[KEY_IMAGES] = images
        return bool(images) and config.search_immediately

    # 指令检测在下方做了
    # 不太方便做是否回复 Bot 自己消息的判断，阉了吧
    return (not config.search_in_group_only_keyword) and (
        ev.is_tome()
        or any(True for x in msg if isinstance(x, At) and x.target == bot.self_id)
    )


async def extract_search_args() -> SearchArgs:
    ev = current_event.get()
    m = current_matcher.get()
    state = m.state

    args = SearchArgs()

    async def finish_with_unknown(arg: str) -> NoReturn:
        await m.finish(
            f"意外参数 {arg}\n使用指令 `{config.search_keyword} -h` 查看帮助",
        )

    async def parse_mode(arg: str):
        if arg.startswith("--") and (mode := arg[2:]) in registered_search_func:
            args.mode = mode
            return True
        return False

    async def is_purge(arg: str):
        if arg == "--purge":
            args.purge = True
            return True
        return False

    async def send_help(arg: str):
        if arg in ("-h", "--help"):
            await UniMessage.image(
                raw=(Path(__file__).parent / "res" / "usage.jpg").read_bytes(),
            ).finish(reply_to=True)
        return True

    msg = (
        _command_arg(state).extract_plain_text()
        if _command(state)
        else ev.get_plaintext()
    )
    for arg in msg.strip().lower().split():
        for func in (parse_mode, is_purge, send_help):
            if await func(arg):
                break
        else:
            await finish_with_unknown(arg)

    return args


async def get_images_from_ev(msg: UniMessage) -> List[Image]:
    m = current_matcher.get()
    state = m.state

    images = state.get(KEY_IMAGES) or await extract_images(msg)
    if images:
        return images

    @waiter(waits=["message"], keep_session=True)
    async def wait_msg(msg: UniMsg):
        return msg

    waited_msg = await wait_msg.wait(
        f"请在 {config.wait_for_image_timeout} 秒内发送你要搜索的图片，发送其他内容取消搜索",
    )
    if not waited_msg:
        await m.finish("操作超时，退出搜图")

    images = await extract_images(waited_msg)
    if not images:
        await m.finish("无效输入，退出搜图")

    return images


@asynccontextmanager
async def fail_with_msg(msg: Union[UniMessage, str], should_finish: bool = True):
    try:
        yield
    except Exception as e:
        logger.exception("Error occurred")
        asyncio.create_task(
            logger.catch(
                (msg if isinstance(msg, UniMessage) else UniMessage(msg)).send,
            )(reply_to=True),
        )
        if should_finish:
            raise FinishedException from e
    return


async def should_display_favorite(target: Target) -> bool:
    return (await SUPERUSER(current_bot.get(), current_event.get())) and target.private


async def send_msgs(
    msgs: List[UniMessage],
    target: Target,
    index: Optional[int] = None,
    display_fav: bool = False,
):
    def pre_process_msg(m: UniMessage):
        if index:
            m = UniMessage.text(f"第 {index} 张图片的搜索结果：\n") + m

        should_remove: List[str] = []
        if not display_fav:
            should_remove.append("❤️ 已收藏\n")
        for txt in should_remove:
            if seg := next((x for x in m[Text] if (txt in x.text)), None):
                seg.text = seg.text.replace(txt, "")

        return m

    msgs = [pre_process_msg(m) for m in msgs]
    msg_len = len(msgs)
    reply_to: Optional[str] = UniMessage.get_message_id()

    async def try_send():
        if config.forward_search_result and msg_len > 1:
            bot = current_bot.get()
            nodes = [
                CustomNode(
                    uid=bot.self_id,
                    name=(next(iter(config.nickname), None) or "YetAnotherPicSearch"),
                    content=x,
                )
                for x in msgs
            ]
            with suppress(SerializeFailed, ActionFailed):
                await UniMessage(Reference(nodes=nodes)).send(
                    target=target,
                    fallback=FallbackStrategy.forbid,
                    # reply_to=reply_to,
                )
                return

        for x in msgs:
            await x.send(
                target=target,
                fallback=FallbackStrategy.to_text,
                reply_to=reply_to,
            )

    with suppress(ActionFailed):
        await try_send()
        return

    target.private = True
    reply_to = None
    with suppress(ActionFailed):
        await try_send()
        return

    with logged_suppress("ActionFailed", ActionFailed):
        await UniMessage.text(
            "消息发送失败了呜呜喵，似乎是被某种神秘的力量拦截了喵",
        ).send(reply_to=True)


@overload
def make_cache_key(mode: str, seg: Image, raw: bytes) -> str: ...
@overload
def make_cache_key(
    mode: str,
    seg: Image,
    raw: Literal[None] = None,
) -> Optional[str]: ...
def make_cache_key(mode: str, seg: Image, raw: Optional[bytes] = None) -> Optional[str]:
    if seg.id:
        adapter_name = current_bot.get().adapter.get_name()
        base = f"id_{adapter_name}_{seg.id}"
    elif raw:
        base = f"hash_{hash(raw):x}"
    else:
        return None
    return f"{hash(f'{base}_{mode}'):x}"


async def handle_single_image(
    client: AsyncClient,
    seg: Image,
    mode: str,
    purge: bool,
    target: Target,
    index: Optional[int] = None,
    display_fav: bool = False,
):
    async def fetch_image(seg: Image) -> Optional[bytes]:
        async with fail_with_msg(
            f"图片{f' {index} ' if index else ''}下载失败",
            should_finish=False,
        ):
            return await get_image_from_seg(seg)
        return None

    # lazy fetch file if cache does not exist
    file = None
    cache_key = make_cache_key(mode, seg)
    if not cache_key:
        if not (file := await fetch_image(seg)):
            return
        cache_key = make_cache_key(mode, seg, file)

    if (not purge) and (cached_msgs := msg_cache.get(cache_key)):
        msgs = [(UniMessage.text("[缓存] ") + x) for x in cached_msgs]
        await send_msgs(msgs, target, index, display_fav)
        return

    # lazy fetch
    if (not file) and (not (file := await fetch_image(seg))):
        return
    file = post_image_process(file)

    messages: List[UniMessage] = []
    func = registered_search_func[mode].func
    while True:
        res = await func(file, client, mode)
        msgs, func = res if isinstance(res, tuple) else (res, None)
        messages.extend(msgs)
        await send_msgs(msgs, target, index, display_fav)
        if not func:
            break

    msg_cache[cache_key] = messages


async def search_handler(msg: UniMsg, target: MsgTarget):
    arg = await extract_search_args()
    images = await get_images_from_ev(msg)

    async with RecallContext() as recall:
        await recall.send("正在进行搜索，请稍候", reply_to=True)

        display_fav = await should_display_favorite(target)
        network = (
            Network(proxies=config.proxy, cookies=config.exhentai_cookies, timeout=60)
            if arg.mode == "ex"
            else Network(proxies=config.proxy)
        )
        multiple_images = len(images) > 1
        async with network as client:
            for index, seg in enumerate(images, 1):
                async with fail_with_msg(
                    f"第 {index} 张图搜索失败",
                    should_finish=False,
                ):
                    await handle_single_image(
                        client,
                        seg,
                        arg.mode,
                        arg.purge,
                        target,
                        index if multiple_images else None,
                        display_fav,
                    )


matcher_search_cmd = on_command(config.search_keyword, priority=1, block=True)
matcher_search_cmd.handle()(search_handler)
if not config.search_keyword_only:
    matcher_search_msg = on_message(rule=rule_func_search_msg, priority=2, block=True)
    matcher_search_msg.handle()(search_handler)
